#include <cscore_cpp.h>
#include <cscore_oo.h>
#include <cscore_oo.inl>
#include <Feedback.h>
#include <llvm/StringRef.h>
#include <stdarg.h>
#include <stdio.h>
#include <SmartDashboard/SmartDashboard.h>
#include <cstring>
#include <vector>

#define USE_TWO_CAMERAS

using namespace cs;

static bool camTwoIsPresent()
{
#ifdef CAMERA
	std::vector<UsbCameraInfo>  camInfos = UsbCamera::EnumerateUsbCameras();
	for (int i = 0; i < camInfos.size(); ++i) {
		printf("Found camera #%d named %s at path %s\n", camInfos[i].dev, camInfos[i].name.c_str(), camInfos[i].path.c_str());
		if (camInfos[i].dev == 1)
			return true;
	}
#else


	return false;
#endif
}

static bool camOneIsPresent()
{
#ifdef CAMERA
	std::vector<UsbCameraInfo>  camInfos = UsbCamera::EnumerateUsbCameras();
	for (int i = 0; i < camInfos.size(); ++i) {
		printf("Found camera #%d named %s at path %s\n", camInfos[i].dev, camInfos[i].name.c_str(), camInfos[i].path.c_str());
		if (camInfos[i].dev == 0)
			return true;
	}

	return false;
#else

	return false;
#endif
}


Feedback::Feedback() :
#ifdef CAMERA
				camServer(NULL),
#endif
		frontCam(true), hasBackCam(false), camBrightness(50),
					   camExposure(CAM_AUTO), camWhiteBalance(0)
{
#ifdef CAMERA
	camServer = new MjpegServer("server", 8888);

	strcpy(wbSettings[0].name, "auto");
	wbSettings[0].value = (VideoCamera::WhiteBalance)CAM_AUTO;
	strcpy(wbSettings[1].name, "fi");
	wbSettings[1].value = VideoCamera::kFixedIndoor;
	strcpy(wbSettings[2].name, "fo1");
	wbSettings[2].value = VideoCamera::kFixedOutdoor1;
	strcpy(wbSettings[3].name, "fo2");
	wbSettings[3].value = VideoCamera::kFixedOutdoor2;
	strcpy(wbSettings[4].name, "ff1");
	wbSettings[4].value = VideoCamera::kFixedFluorescent1;
	strcpy(wbSettings[5].name, "ff2");
	wbSettings[5].value = VideoCamera::kFixedFlourescent2;

	curWbSetting = wbSettings + 4;
	camWhiteBalance = curWbSetting->value;
#endif
}

void Feedback::initCameras() {
#ifdef CAMERA
	std::vector<UsbCameraInfo>  camInfos = UsbCamera::EnumerateUsbCameras();


	std::vector<std::string> v;

	for (int i = 0; i < camInfos.size(); ++i) {
		printf("Found camera #%d named %s at path %s\n", camInfos[i].dev, camInfos[i].name.c_str(), camInfos[i].path.c_str());
		if (camInfos[i].dev == 0) {
			v.push_back(camInfos[i].path);
#ifdef USE_TWO_CAMERAS
		} else if (camInfos[i].dev == 1) {
			v.push_back(camInfos[i].path);
			hasBackCam = true;
#endif
		}
	}


	// defaults seem to be: Auto white bal; exposure 50%; brigthness 80%
	//
	// axis was: color level 70; brightness 50; sharpness 0; white balance Fixed Fluorescent 1; exposure Auto; no exposure priority
	for (size_t i = 0; i < v.size(); ++i) {
		UsbCamera *camera =	new UsbCamera(i == 0 ? "cam0" : "cam1", v[i]);

		camera->SetExposureAuto();
		camera->SetBrightness(50);
		camera->SetWhiteBalanceManual(VideoCamera::kFixedFluorescent1);
		camVec.push_back(camera);
	}
	camServer->SetSource(*(camVec[0]));
	send_Debug_Double("thunderdashboard", "frontcamera", frontCam ? 1 : 0);


#endif
}


void Feedback::processCameraSettings()
{
#ifdef CAMERA
	int n = (int)SmartDashboard::GetNumber("camera_brightness", camBrightness);
	if (n != camBrightness) {
		camBrightness = n;
		if (n > 100)
			n = 100;
		if (n < 0)
			n = 0;
		for (size_t i = 0; i < camVec.size(); ++i) {
			camVec[i]->SetBrightness(n);
		}
	}

	n = (int)SmartDashboard::GetNumber("camera_exposure", camExposure);
	if (n != camExposure) {
		camExposure = n;
		if (n > 100)
			n = 100;
		if (n < 0 && n != CAM_AUTO)
			n = 0;

		for (size_t i = 0; i < camVec.size(); ++i) {
			if (n == CAM_AUTO)
				camVec[i]->SetExposureAuto();
			else
				camVec[i]->SetExposureManual(n);
		}
	}

	std::string ns = SmartDashboard::GetString("camera_whitebalance", curWbSetting->name);
	if (ns.compare(curWbSetting->name) != 0) {
		for (size_t i = 0; i < 6; ++i) {
			if (ns.compare(wbSettings[i].name) == 0) {
				curWbSetting = wbSettings + i;

				for (size_t i = 0; i < camVec.size(); ++i) {
					if (curWbSetting->value == CAM_AUTO)
						camVec[i]->SetWhiteBalanceAuto();
					else
						camVec[i]->SetWhiteBalanceManual(curWbSetting->value);
				}
			}
		}
	}

#endif
}

void Feedback::send_Debug_String(const char *subsystemName, const char *dataName, const char *format, ...) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
	  char buffer[256]; //sets the maximum character count to 256
	  va_list args; // args is the "..."
	  va_start (args, format); //marks the start of the list of arguments(args)
	  vsnprintf (buffer,256,format, args); //printing the args after the keyName
	  va_end (args);//marks end of arguments (this allows us to change the arguments and not the keyname)
	  SmartDashboard::PutString(keyName.c_str(), buffer);//this prints keyName into the table, then the arguments
	}

void Feedback::send_Debug_Double(const char *subsystemName, const char *dataName, double format){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	SmartDashboard::PutNumber(keyName.c_str(), format); //this prints keyName and the data into the table
}

void Feedback::send_Editable_String(const char *subsystemName, const char *dataName, const char *format, ...) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
		keyName += "_e"; // End in _e to make editable
	  char buffer[256]; //sets the maximum character count to 256
	  va_list args; // args is the "..."
	  va_start (args, format); //marks the start of the list of arguments(args)
	  vsnprintf (buffer,256,format, args); //printing the args after the keyName
	  va_end (args);//marks end of arguments (this allows us to change the arguments and not the keyname)
	  SmartDashboard::PutString(keyName.c_str(), buffer);//this prints keyName into the table, then the arguments
	}

void Feedback::send_Editable_Double(const char *subsystemName, const char *dataName, double format){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	keyName += "_e"; // End in _e to make editable
	SmartDashboard::PutNumber(keyName.c_str(), format); //this prints keyName and the data into the table
}


std::string Feedback::get_Editable_String(const char *subsystemName, const char *dataName, llvm::StringRef defaultVal) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
		keyName += "_e"; // End in _e to make editable
	  return SmartDashboard::GetString(keyName.c_str(), defaultVal);//this prints keyName into the table, then the arguments
	}

double Feedback::get_Editable_Double(const char *subsystemName, const char *dataName, double defaultVal){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	keyName += "_e"; // End in _e to make editable
	return SmartDashboard::GetNumber(keyName.c_str(), defaultVal);
}



void Feedback::swapCamera()
{
#ifdef CAMERA
#ifdef USE_TWO_CAMERAS
	if (hasBackCam) {
		UsbCamera *cam = camVec[frontCam ? 0 : 1];

        frontCam = !frontCam;
        camServer->SetSource(*cam);
	}
#endif
	send_Debug_Double("thunderdashboard", "frontcamera", frontCam ? 1 : 0);

#endif
}

bool Feedback::isFrontCamera()
{
	return (frontCam);

}


